-- name=int_clues_panel
--******************************************************************************************
function public.Init()

  private.MOUSE_OVER_ANIM_TIME = 0.3;
  private.SHOW_ANIM_TIME       = 0.6;
  private.SHOWED_TIME          = 3;

  private.PANEL_MOVE = {1015, 558};
  private.HEAD_CLUE_POS = {-215, -233};
  private.CLUES_X = -276;
  private.CLUE_POS_Y_STEP = 12;
  private.CLUE_POS_X_STEP = 22;
  
  private.clues_shown = false;
  private.anim_progress = false;
  private.texts_size = 0;
  
  public.clues_pull = {};

  ObjAttach("int_clues_panel", interface.originhub);
  
  --public.AddCluesToPanel( ng_global.clues );
  --public.SetCrossedClues( ng_global.clues_found );

end;
--******************************************************************************************
-------------------------------------------------------------------------------------
function public.SetCluesPanelActive()
  
  ObjSet("obj_int_inventory_btn_clues",  { alp = 1, input = 1 });
  
end;
---------------------------------------------------------------------------------------------------------------------------------------------------------------------
function public.ActivateAnim( func_after_anim )
  
  func_after_anim = func_after_anim or false;
  
  local func_after = function()
    
    public.SetCluesPanelActive();
    
    if func_after_anim then
      func_after_anim();
    end;
    
  end;
  
  ObjAnimate("obj_int_inventory_btn_clues", "alp", 0, 0, func_after, { 0, 0, 0,   0.5, 0, 1 });
  
end;
------------------------------------------------------------------------------------
function public.AddCluesToPanel( clues )
  
  for i = 1, #clues, 1 do
    
    ObjCreate("obj_int_clues_panel_text_"..clues[i], "obj");
    ObjSet("obj_int_clues_panel_text_"..clues[i],  { pos_x = private.CLUES_X,
                                                     pos_y = 0,
                                                     input = 0 });
                                                   
    ObjAttach("obj_int_clues_panel_text_"..clues[i], "obj_int_clues_panel");
    
    ObjCreate("txt_int_clues_panel_text_"..clues[i].."_num", "text");
    ObjSet("txt_int_clues_panel_text_"..clues[i].."_num",  { pos_x = 0, 
                                                             pos_y = 0,
                                                             res = "assets/fonts/times",
                                                             text = i;
                                                             align = 0,
                                                             fontsize = 20,
                                                             maxwidth = 20,
                                                             fontcolor_r = 0.239216,
                                                             fontcolor_g = 0.117647,
                                                             fontcolor_b = 0.0431373,
                                                             display_outline = true,
                                                             outline_size = 1,
                                                             input = 0 });

    ObjAttach("txt_int_clues_panel_text_"..clues[i].."_num", "obj_int_clues_panel_text_"..clues[i]);
    
    ObjCreate("txt_int_clues_panel_text_"..clues[i], "text");
    ObjSet("txt_int_clues_panel_text_"..clues[i],  { pos_x = private.CLUE_POS_X_STEP, 
                                                     pos_y = 0,
                                                     res = "assets/fonts/times",
                                                     text = StringGet("str_int_clues_panel_text_"..clues[i]);
                                                     align = 0,
                                                     fontsize = 18,
                                                     maxwidth = 110,
                                                     fontcolor_r = 0.239216,
                                                     fontcolor_g = 0.117647,
                                                     fontcolor_b = 0.0431373,
                                                     input = 0,
                                                     alp = 0 });

    ObjAttach("txt_int_clues_panel_text_"..clues[i], "obj_int_clues_panel_text_"..clues[i]);
    
    ObjCreate("spr_int_clues_panel_crossed_"..clues[i], "spr");
    ObjSet("spr_int_clues_panel_crossed_"..clues[i],  { pos_x = 0, 
                                                        pos_y = 0,
                                                        res = "assets/interface/resources/clues_panel/icon_"..clues[i],
                                                        input = 0, 
                                                        alp = 1 });
                                                      
    ObjAttach("spr_int_clues_panel_crossed_"..clues[i], "txt_int_clues_panel_text_"..clues[i]);
    
  end;
  
  private.MoveCluesJournal();
  
end;
------------------------------------------------------------------------------------
function public.ShowNewClues( new_clues, func)
  
  if #new_clues > 0 then
    
    local func_after = function ( nc )
      
      for i = 1, #nc, 1 do
        ObjAnimate("obj_int_clues_panel_text_"..nc[i], "alp", 0, 0, "", { 0, 0, 0,   0.5, 0, 1 });
      end;
      
    end;
    
    for i = 1, #new_clues, 1 do
      
      ObjSet("obj_int_clues_panel_text_"..new_clues[i],  { alp = 0 });
      
    end;
    
    if not private.clues_shown then
      public.ShowAnim( function() func_after(new_clues); end, func );
    else 
      func_after(new_clues);
      ObjSet("tmr_int_clues_panel", { time = private.SHOWED_TIME, playing = 1, endtrig = function() public.HideAnim(func); end });
    end;
      
  elseif func then
    func();
  end;
  
end;
------------------------------------------------------------------------------------
function private.MoveCluesJournal()

  local text_size = private.HEAD_CLUE_POS[2] + math.floor( ObjGet("txt_int_clues_panel_head").draw_height/2 );

  for i = 1, #ng_global.clues, 1 do

    local size_h = ObjGet("txt_int_clues_panel_text_"..i).draw_height;
    local size_w = ObjGet("txt_int_clues_panel_text_"..i).draw_width;
    local old_size_h = text_size + private.CLUE_POS_Y_STEP;
    text_size = text_size + size_h + private.CLUE_POS_Y_STEP;
    
    --ObjSet("obj_int_clues_panel_text_"..i, { pos_y = old_size_h });
    
    ObjMultiSet( { {"obj_int_clues_panel_text_"..i, { pos_y = old_size_h }},
                   {"spr_int_clues_panel_crossed_"..i, { pos_x = size_w + 25 }} } );

  end;

  private.texts_size = text_size;  

end;
------------------------------------------------------------------------------------
function public.SetCrossedClues( clues )
  
  for i = 1, #clues, 1 do
    
    ObjSet("txt_int_clues_panel_text_"..clues[i],  { alp = 1 });
    
  end;
  
end;
------------------------------------------------------------------------------------
function public.CrossClues( new_clues, func )
  
  if #new_clues > 0 then
    
    local func_after = function ( nc )
      
      for i = 1, #nc, 1 do
        ObjAnimate("txt_int_clues_panel_text_"..nc[i], "alp", 0, 0, "", { 0, 0, 0,   0.3, 0, 1 });
      end;
      
    end;
    
    for i = 1, #new_clues, 1 do
      
      ObjSet("txt_int_clues_panel_text_"..new_clues[i],  { alp = 0 });
      
    end;
    
    if not private.clues_shown then
      public.ShowAnim( function() func_after(new_clues); end, func );
    else 
      func_after(new_clues);
      ObjSet("tmr_int_clues_panel", { time = private.SHOWED_TIME, playing = 1, endtrig = function() public.HideAnim(func); end });
    end;
      
  elseif func then
    func();
  end;
  
end;
---------------------------------------------------------------------------------------------------------------------------------------------------------------------
function public.ShowAnim( func_after_anim, func_after_hide )

  if not private.clues_shown then
    
    cmn.PlayAudio("snd", "assets/levels/common/audio/aud_clk_paper");
    
    private.clues_shown = true;
    private.anim_progress = true;
    
    ObjMultiSet( { {"spr_int_clues_panel",  { alp = 1 }},
                   {"tmr_int_clues_panel", { time = private.SHOWED_TIME, playing = 1, endtrig = function() public.HideAnim(func_after_hide); end }} } );

    local func_after = function ()
      
      private.anim_progress = false;

      if func_after_anim then
        func_after_anim();
      end;

    end;

    local tme_show = private.SHOW_ANIM_TIME;
    local pos_panel_y = ObjGet("spr_int_clues_panel").pos_y;
    
    ObjAnimate( "spr_int_clues_panel", "pos_y", 0, 0, func_after,
    {
      0.0, 2, pos_panel_y,
      tme_show, 2, private.PANEL_MOVE[2]
    } );
    
  elseif not private.anim_progress then
    public.HideAnim();
  end;

end;
------------------------------------------------------------------------------------
function public.HideAnim( func_after_hide )

  if private.clues_shown then

    --cmn.Lock(1);
    private.clues_shown = false;
    private.anim_progress = true;

    local func_after = function ()
      
      private.anim_progress = false;
      
      ObjSet("spr_int_clues_panel",  { alp = 0 });
      
      if func_after_hide then
        func_after_hide();
      end;
      
    end;

    local tme_show = private.SHOW_ANIM_TIME;
    local pos_panel_y = ObjGet("spr_int_clues_panel").pos_y;

    ObjAnimate( "spr_int_clues_panel", "pos_y", 0, 0, func_after,
    {
      0.0, 2, pos_panel_y,
      tme_show, 2, private.PANEL_MOVE[1]
    } );

  end;

end;
-----------------------------------------------------------------------------