-- name=int_taskpanel
--******************************************************************************************
function public.Init( pos_z )

  local impl_module_name = nil;

  if ( _G[ "int_taskheader" ] ) then

    impl_module_name = "int_taskheader_taskpanel_impl";

  end;

  interface.LoadImplementation( "taskpanel", impl_module_name );
  ObjAttach( "int_taskpanel", interface.originhub );
  ObjSet( "int_taskpanel", { pos_z = pos_z } );
  private.event_anim_end = {};
  private.m_tasks_data = nil;
  public.already_hide = nil;

  MsgSubscribe( Command_Interface_UpdateTextures, private.WideScreenUpdate );
  MsgSubscribe( Command_TaskPanel_DecreaseZ, private.DecreaseZ );
  MsgSubscribe( Command_TaskPanel_IncreaseZ, private.IncreaseZ );
  MsgSubscribe( Event_Level_CheatKeyPressed, private.HotKey );

end;
--******************************************************************************************
function public.Destroy()

  MsgUnsubscribe( Command_Interface_UpdateTextures, private.WideScreenUpdate );
  MsgUnsubscribe( Command_TaskPanel_DecreaseZ, private.DecreaseZ );
  MsgUnsubscribe( Command_TaskPanel_IncreaseZ, private.IncreaseZ );
  MsgUnsubscribe( Event_Level_CheatKeyPressed, private.HotKey );

end;
--******************************************************************************************
function public.EventAnimEnd( event_id )

  --DbgTrace( "***int_taskpanel::public.EventAnimEnd: event_id = "..event_id );

  local event_params = private.event_anim_end[ event_id ];

------------------------------------------------------------------------------------
  if ( event_id == "show" ) then

  ObjSet( "int_taskpanel", { input = true } );

------------------------------------------------------------------------------------
  elseif ( event_id == "hide" ) then

  ObjSet( "int_taskpanel", { active = false, visible = false } );

------------------------------------------------------------------------------------
  elseif ( event_id == "taskheader_show" ) then

  ObjSet( "int_taskpanel", { input = true } );

------------------------------------------------------------------------------------
  elseif ( event_id == "taskheader_hide" ) then

  ObjSet( "int_taskpanel", { active = false, visible = false } );

------------------------------------------------------------------------------------
  elseif ( string.find( event_id, "unmark_task" ) ) then

  local task_name   = event_params[ "name" ];

  local task_id     = public.task_id[ task_name ];
  local task_params = public.task_params[ task_id ];

  int_taskpanel_impl.SetTask( task_id, task_params );

  elseif ( string.find( event_id, "mark_task" ) ) then

  local task_name   = event_params[ "name" ];

  local task_id     = public.task_id[ task_name ];
  local task_params = public.task_params[ task_id ];

  int_taskpanel_impl.SetTask( task_id, task_params );

------------------------------------------------------------------------------------
  elseif ( string.find( event_id, "complete_task" ) ) then

    local task_name   = event_params[ "name" ];
    --local object_name = event_params[ "game_name" ];
    --local task_count  = event_params[ "count" ];
    local open_panel_door = event_params[ "open_panel_door" ];

    local task_id     = public.task_id[ task_name ];
    local task_params = public.task_params[ task_id ];

    --DbgTrace( "***int_taskpanel.EventAnimEnd: event_id = "..event_id..", task_name = "..task_name..", open_panel_door = "..open_panel_door );

    if ( int_taskpanel_impl.IsTaskImg( task_name ) ) then
      
      ObjDelete( "spr_"..common.GetObjectName( task_name ).."_task_img" );
      
    else
    
      int_taskpanel_impl.SetTask( task_id, task_params );
      
    end;

    if private.m_tasks_data ~= nil then

      local all_collected = true;

      for i = 1, #private.m_tasks_data do
        if private.m_tasks_data[ i ].name == task_name then
          local collected = private.m_tasks_data[ i ].collected;
          local count = private.m_tasks_data[ i ].count;
          if count == collected then
            private.m_tasks_data[ i ].completed = 1;
          end;
        end;
        if private.m_tasks_data[ i ].completed ~= 1 then
          all_collected = false;
        end;
      end;

      if ( open_panel_door ~= 0 ) then
        int_taskpanel_impl.OpenPanelDoorAnim( open_panel_door, 2, room_name );
      end;

      if all_collected then
        MsgSend( Event_DialogHo_Show, {} );
      end;

    end;

  end;
------------------------------------------------------------------------------------

end;
--******************************************************************************************
function public.Switch()

  if ( _G[ "int_taskheader" ] ) then

    int_taskheader.MouseDown( "taskpanel" );

  end;

end;
--******************************************************************************************
function public.ShowForTaskHeader( ho_tasks )

  public.Show( ho_tasks, true );

end;
--******************************************************************************************
function public.HideForTaskHeader()

  public.Hide( true );

end;
--******************************************************************************************
function public.Show( ho_tasks, is_for_taskheader, changetask)

  --DbgTrace( "***int_taskpanel.Show: changetask = "..tostring(changetask) );

  if private.m_tasks_data then private.m_tasks_data = nil; end;

  private.m_tasks_data = ho_tasks;

  if (private.m_tasks_data) then

    for i = 1, #private.m_tasks_data do
      private.m_tasks_data[ i ].collected = 0;
      private.m_tasks_data[ i ].completed = 0;
      if private.m_tasks_data[ i ].count == 0 then
        private.m_tasks_data[ i ].completed = 1;
      end;
    end;

  end;

  if ( not changetask ) then

    public.already_hide = false;

    ObjSet( "int_taskpanel", { active = true, visible = true } );

    --DbgTrace( "tasks count == "..#ho_tasks );

    public.task_id     = {};
    public.task_params = {};

    for i = 1, #ho_tasks, 1 do

      local task_name   = ho_tasks[ i ][ "name"   ]
      local task_count  = ho_tasks[ i ][ "count"  ];
      local task_marked = ho_tasks[ i ][ "marked" ];

      public.task_id[ task_name ] = i;
      public.task_params[ i ] =
      {
        name   = task_name,
        count  = task_count,
        marked = task_marked
      };

      if ( int_taskpanel_impl.IsTaskImg( task_name ) ) then
        int_taskpanel_impl.SetTaskImg( i, task_name );
      else
        int_taskpanel_impl.SetTask( i, public.task_params[ i ] );
      end;
      
    end;

    for i = #ho_tasks + 1, 12, 1 do

      ObjStopAnimate( "obj_int_taskpanel_task_"..i.."_line", 8 );

      ObjSet( "txt_int_taskpanel_task_"..i, { alp = 0 } );
      ObjSet( "obj_int_taskpanel_task_"..i.."_line", { alp = 0 } );

    end;

    int_taskpanel_impl.SetTaskPos( #ho_tasks );

    private.SetPanelDoorTasks( ho_tasks );

    if ( is_for_taskheader == nil ) then

      local event_id = "show";
      private.event_anim_end[ event_id ] = {};
      int_taskpanel_impl.ShowAnim( event_id );

    elseif ( is_for_taskheader ) then

      local event_id = "taskheader_show";
      private.event_anim_end[ event_id ] = {};
      int_taskpanel_impl.TaskHeaderShowAnim( event_id );

    end;

  else

    int_taskpanel_impl.HO_TASK_NEW = ho_tasks;

    int_taskpanel_impl.SetNewTaskAnim();

  end;

end;
--******************************************************************************************
function public.CompleteTask( task_name, game_name )

  --DbgTrace("***int_taskpanel::public.CompleteTask: task_name = "..task_name..", game_name = "..game_name);

  local answer = MsgSend( Request_Level_IsLoading, {} );
  if answer.loading == 1 then

    ObjDelete( game_name );
    game_name = "";    

  end;

  if game_name ~= "" then

    ObjSet( game_name, { input = false } );

  end;

  local task_count = 0;

  for i = 1, #private.m_tasks_data do

    local collected = private.m_tasks_data[ i ].collected;
    local count = private.m_tasks_data[ i ].count;

    if private.m_tasks_data[ i ].name == task_name then
      if collected < count then
        private.m_tasks_data[ i ].collected = collected + 1;
        collected = collected + 1;
        task_count = count - collected;
      end;    
    end;

  end;

  local open_panel_door = public.CompletePanelDoorTask();

  local task_id = public.task_id[ task_name ];
  public.task_params[ task_id ].count = task_count;

  if ( game_name ) then

    -- блок исключен за отсуствием необходимости
    -- мешает реализации анимаций с частицами по маске
    --[[
    local task_obj_name = "obj_int_taskpanel_task_"..task_id;
    local item_obj_name = "fly_int_taskpanel_item_"..task_id.."_"..task_count;

    local item_pos = GetObjPosByObj( game_name );
    
    ObjCreate( item_obj_name, "obj" );
    ObjSet( item_obj_name,
    {
      pos_x = item_pos[ 1 ],
      pos_y = item_pos[ 2 ],
      pos_z = 13
    } );
    ObjAttach( item_obj_name, InterfaceWidget_Top_Name );

    ObjSet( game_name, { pos_x = 0, pos_y = 0, pos_z = 13 } );
    ObjAttach( game_name, item_obj_name );

    local event_id = "complete_task_"..task_id.."_"..task_count;
    private.event_anim_end[ event_id ] = message_params;

    interface.taskpanel.Switch();
    interface.taskpanel.CompleteTaskAnim( item_obj_name, task_id, event_id );
    ]]

    local event_id = "complete_task_"..task_id.."_"..task_count;
    private.event_anim_end[ event_id ] = { name = task_name, game_name = game_name, task_count = count, open_panel_door = open_panel_door };

    public.Switch();
    int_taskpanel_impl.CompleteTaskAnim( task_name, game_name, task_id, task_count, event_id );

  end;

end;
--******************************************************************************************
function public.SetTaskMark( task_name, mark_state )
 
  local task_id = public.task_id[ task_name ];

  public.task_params[ task_id ].marked = mark_state;

  if     ( mark_state == 0 ) then

    local event_id = "unmark_task_"..task_name;
    private.event_anim_end[ event_id ] = { name = task_name, mark = mark_state };

    --interface.taskpanel.Switch();
    int_taskpanel_impl.UnmarkTaskAnim( task_id, event_id );

  elseif ( mark_state == 1 ) then

    local event_id = "mark_task_"..task_name;
    private.event_anim_end[ event_id ] = { name = task_name, mark = mark_state };

    --interface.taskpanel.Switch();
    int_taskpanel_impl.MarkTaskAnim( task_id, event_id );

  end;

end;
--******************************************************************************************
function public.ShowHoHint()

  --DbgTrace( "***int_taskpanel::public.ShowHoHint" );

  if private.m_tasks_data == nil then return; end;

  local not_objects_task_counts = 0;
  local object_tasks = {};
  for i = 1, #private.m_tasks_data do
    if private.m_tasks_data[ i ].collected < private.m_tasks_data[ i ].count then

      local hint_objects = {};
      local hint = private.m_tasks_data[ i ].hint;
      for j = 1, #hint do
        for k = 1, #hint[ j ] do
          local params = ObjGet( hint[ j ][ k ] );
          if params ~= nil and params.input then
            table.insert( hint_objects, hint[ j ][ k ] );
          else
            hint_objects = {};
            break;
          end; 
          if #hint_objects > 0 then
            break;
          end;
        end;
        if #hint_objects > 0 then
          table.insert( object_tasks, hint_objects );
        else
          not_objects_task_counts = not_objects_task_counts + 1;
        end;
      end;

    end;
  end;

  if #object_tasks == 0 and not_objects_task_counts == 0 then

    MsgSend( Event_HoHint_NoObjects, {} );

  else

    if #object_tasks > 0 then

      local index = math.random( 1, #object_tasks );
      local hint_table = { hint = {} };
      for i = 1, #object_tasks[ index ] do
        local objname = object_tasks[ index ][ i ];
        local pos = GetObjPosByObj( objname );
        --DbgTrace( pos[ 1 ].." "..pos[ 2 ].." "..objname );
        table.insert( hint_table.hint, { pos_x = pos[ 1 ], pos_y = pos[ 2 ], obj_name = objname } );
      end;
      MsgSend( Command_Effect_ShowHoHint, hint_table );

    else
      MsgSend( Event_HoHint_NoObjects, {} );
    end;

  end;

end;
--******************************************************************************************
function public.Hide( for_taskpanel )

  if ( not public.already_hide ) then

    public.already_hide = true;
    private.m_tasks_data = nil;

    ObjSet( "int_taskpanel", { input = false } );

    if ( not for_taskpanel ) then

      local event_id = "hide";
      private.event_anim_end[ event_id ] = {};
      int_taskpanel_impl.HideAnim( event_id );

    elseif ( for_taskpanel ) then
   
      local event_id = "taskheader_hide";
      private.event_anim_end[ event_id ] = {};
      int_taskpanel_impl.TaskHeaderHideAnim( event_id );

    end;

  end;

end;
--******************************************************************************************
function public.StdFlyAnimation( item_obj_name, task_obj_name, trigger_end, reduced_time )

  reduced_time = reduced_time or 0;

  SendMessage( Command_Transporter_MoveObject,
  {
    type       = Trans_SceneToTaskPanel,
    obj_name   = item_obj_name,
    toobj_name = task_obj_name,
    end_trig   = trigger_end,
    red_time   = reduced_time
  } );

end;
--******************************************************************************************
function private.HotKey( msg, params )

  if params.key == 13 --[[Enter - key]] then  

  elseif params.key == 32 --[[Space - key]] then

    --MsgSend( Event_DialogHo_Show, {} );
    common_impl.DialogHo_Show( true );

  end;

end;
--******************************************************************************************
function private.WideScreenUpdate()

  interface.WideScreenUpdate( "taskpanel" );

end;
--******************************************************************************************
function private.StdFlyAnimation( item_obj_name, task_obj_name, trigger_end, reduced_time )

  reduced_time = reduced_time or 0;

  SendMessage( Command_Transporter_MoveObject,
  {
  type       = Trans_SceneToTaskPanel,
  obj_name   = item_obj_name,
  toobj_name = task_obj_name,
  end_trig   = trigger_end,
  red_time   = reduced_time
  } );

end;
--******************************************************************************************
function private.DecreaseZ()
  ObjSet( "int_taskpanel", { pos_z = ObjGet( "int_taskpanel" ).pos_z - 1 } );
end;
--******************************************************************************************
function private.IncreaseZ()
  ObjSet( "int_taskpanel", { pos_z = ObjGet( "int_taskpanel" ).pos_z + 1 } );
end;
--*********************************************************************************************************************
--***function *** PANEL DOORS *** () end*******************************************************************************
--*********************************************************************************************************************
function public.CompletePanelDoorTask()

  local room_name = common.GetObjectName( GetCurrentRoom() );
  local panel_doors = ng_global.progress[ prg.current ][ "win_"..room_name ].panel_doors;

  --DbgTrace( "***int_taskpanel.CompletePanelDoorTask: room_name = "..room_name );

  if ( panel_doors ) then

    local panel_doors_task_complete = { 0, 0, 0, 0 };
    local door_num = 0;

    for i = 1, #private.m_tasks_data, 1 do

      if ( private.m_tasks_data[ i ].completed == 1 ) or ( private.m_tasks_data[ i ].count == private.m_tasks_data[ i ].collected ) then

        door_num = (((i-1)%4)+1);

        --DbgTrace( "panel["..door_num.."]: task["..i.."]: name = "..private.m_tasks_data[ i ].name..", done" );

        panel_doors_task_complete[ door_num ] = panel_doors_task_complete[ door_num ] + 1;

      end;

    end;

    for i = 1, 4, 1 do
      if ( panel_doors_task_complete[i] == 3 ) and ( panel_doors[ i ] == 1 ) then
        panel_doors[ i ] = 2;
        return i;
      end;
    end;

  end;

  return 0;

end;
--******************************************************************************************
function public.OpenPanelDoor( num, trg )

  local room = GetCurrentRoom();
  local room_name = common.GetObjectName( room );
  --DbgTrace( "***int_taskpanel.OpenPanelDoor: room = "..room..", num = "..num..", trg = "..tostring(trg) );

  local panel_doors = ng_global.progress[ prg.current ][ "win_"..room_name ].panel_doors;

  if ( panel_doors ) then

    panel_doors[ num ] = panel_doors[ num ] + 1;

    --DbgTrace( "panel_doors = { "..panel_doors[1]..", "..panel_doors[2]..", "..panel_doors[3]..", "..panel_doors[4].." }" );

    if ( panel_doors[ num ] == 1 ) then

      local ho_tasks = private.m_tasks_data;

      for i = 1, #ho_tasks, 1 do

        if ( (((i-1)%4)+1) == num ) and ( ho_tasks[i].marked == 0 ) then

          for j = 1, #ho_tasks[i].hint, 1 do
            for k = 1, #ho_tasks[i].hint[j], 1 do
              --DbgTrace( "panel["..(((i-1)%4)+1).."]: task["..i.."]: hint["..j.."]["..k.."] == "..ho_tasks[i].hint[j][k]..", input = 1" );
              ObjSet( ho_tasks[i].hint[j][k], { input = 1 } );
            end;
          end;

        end;

      end;

      int_taskpanel_impl.OpenPanelDoorAnim( num, panel_doors[ num ], room_name, trg );

    elseif ( panel_doors[ num ] == 3 ) then

      if ( int_taskpanel_impl.IsAllPanelDoorsItmUsed( room_name ) ) then

        for i = 1, 4, 1 do
          int_taskpanel_impl.OpenPanelDoorAnim( i, 3, room_name );
        end;

        local ho_stage = ng_global.progress[ prg.current ][ "win_"..room_name ].stage;

        if ( ho_stage ) then

          ho_stage[1] = ho_stage[1] + 1;

          if ( _G[ room ][ "stage_"..ho_stage[1] ] ) then
            _G[ room ][ "stage_"..ho_stage[1] ] ();
          end;

        end;

      end;

    end;

  end;

end;
--******************************************************************************************
function private.SetPanelDoorTasks( ho_tasks )

    local room = GetCurrentRoom();
    local room_name = common.GetObjectName( room );

    --DbgTrace( "***int_taskpanel.SetPanelDoorTasks: room = "..room );

    local panel_doors = ng_global.progress[ prg.current ][ "win_"..room_name ].panel_doors;

    if ( panel_doors ) and ( not int_taskpanel_impl.IsAllPanelDoorsItmUsed( room_name ) ) then

      --DbgTrace( "panel_doors = { "..panel_doors[1]..", "..panel_doors[2]..", "..panel_doors[3]..", "..panel_doors[4].." }" );

      for i = 1, #ho_tasks, 1 do

        local door_num = (((i-1)%4)+1);

        if ( panel_doors[ door_num ] == 0 ) then

          for j = 1, #ho_tasks[i].hint, 1 do
            for k = 1, #ho_tasks[i].hint[j], 1 do
              --DbgTrace( "panel ["..door_num.."]: task["..i.."]: hint["..j.."]["..k.."] = "..ho_tasks[i].hint[j][k]..", input = 0" );
              ObjSet( ho_tasks[i].hint[j][k], { input = 0 } );
            end;
          end;

        elseif ( panel_doors[ door_num ] >= 2 ) then

          --DbgTrace( "panel ["..door_num.."]: task["..i.."]: alp = 0" );

          ObjSet( "txt_int_taskpanel_task_"..i, { alp = 0 } );
          ObjSet( "obj_int_taskpanel_task_"..i.."_line", { alp = 0 } );

        end;

      end;

    end;

end;
--******************************************************************************************
