-- name=ho_charmonlock
--*********************************************************************************************************************
function public.Init()
  --------------------------------------------------------------------------------------------------
  room.Define( "ho_charmonlock" );
  game.relations[ "ho_charmonlock" ].exitroom = "rm_housegarden";

  private.item_names = { 
    "eclipse", "letter", "watch", "luck", "music", "aladdin", "magic", "redhood", "hunter", "tritonmermaid", "сinderellacarriage", "clown" 
  };

  public.changed_name = { "", "" };
  private.selected_item = 0;
  ------------------------------------------------------------------------------
  game.charmonlock = {};
  game.charmonlock.double_hint = {
    { "spr_charmonlock_sun",       "spr_charmonlock_moon" },       --1 eclipse
    { "spr_charmonlock_letter",    "spr_charmonlock_pen" },        --2 letter
    { "spr_charmonlock_watch",     "spr_charmonlock_arrow" },      --3 watch
    { "spr_charmonlock_clower",    "spr_charmonlock_horseshoe" },  --4 luck
    { "spr_charmonlock_note",      "spr_charmonlock_trebleclef" }, --5 music
    { "spr_charmonlock_lamp",      "spr_charmonlock_fez" },        --6 aladdin
    { "spr_charmonlock_hat",       "spr_charmonlock_magicwand" },  --7 magic
    { "spr_charmonlock_cloack",    "spr_charmonlock_basket" },     --8 redhood
    { "spr_charmonlock_hathunter", "spr_charmonlock_gun" },        --9 hunter
    { "spr_charmonlock_trident",   "spr_charmonlock_crown" },      --10 tritonmermaid
    { "spr_charmonlock_mouse",     "spr_charmonlock_pumpkin" },    --11 сinderellacarriage
    { "spr_charmonlock_hutclown",  "spr_charmonlock_clownrod" }    --12 clown
  };
  --------------------------------------------------------------------------------------------------
  game.hint[ "win_charmonlock" ] = 
  {
    type = "win",
    room = "rm_housegarden",
    zz = "ho_charmonlock",
    zz_gate = "gho_housegarden_charmonlock"
  };
  --------------------------------------------------------------------------------------------------
  local win_charmonlock_func = function ()

    if ( cmn.IsEventDone( "win_charmonlock" ) ) then
      ObjDelete("gho_housegarden_charmonlock");
      ObjMultiSet( { { "grm_housegarden_castlehall", { input = 1, alp = 1 } },
                     { "anm_housegarden_portal", { animfunc = "func_rm_castlehall_portal" } } } );
      if port.isPortable() == 1 then
        port.GrmEventLightOn("grm_housegarden_castlehall_focus");
		iOS_EnableMultiTouch(1); 
      end;
    end;

  end;
  cmn.AddSubscriber( "win_charmonlock", win_charmonlock_func );
  --------------------------------------------------------------------------------------------------
  local start_charmonlock_func = function ()

    if ( cmn.IsEventStart( "win_charmonlock" ) ) then

      common_impl.ProcessHoStart( "charmonlock" );

    end;

  end;
  cmn.AddSubscriber( "win_charmonlock", start_charmonlock_func, "ho_charmonlock" );
  
  --for secretroom
  private.tasks =  {    
    { "eclipse",  1, 0, { { "obj_charmonlock_eclipse" } } },
    { "letter",  1, 0, { { "obj_charmonlock_letter" } } },
    { "watch",  1, 0, { { "obj_charmonlock_watch" } } },
    { "luck",  1, 0, { { "obj_charmonlock_luck" } } },

    { "music", 1, 0, { { "obj_charmonlock_music" } } },
    { "aladdin", 1, 0, { { "obj_charmonlock_aladdin" } } },
    { "magic", 1, 0, { { "obj_charmonlock_magic" } } },
    { "redhood", 1, 0, { { "obj_charmonlock_redhood" } } },

    { "hunter",  1, 0, { { "obj_charmonlock_hunter" } } },
    { "tritonmermaid",  1, 0, { { "obj_charmonlock_tritonmermaid" } } },
    { "сinderellacarriage",  1, 0, { { "obj_charmonlock_сinderellacarriage" } } },
    { "clown",  1, 0, { { "obj_charmonlock_clown" } } }
  };
    
  private.start = {};
  
  for i = 1, #private.tasks, 1 do

    for j = 1, private.tasks[i][2], 1 do

      table.insert( private.start, {} );
      private.start[ #private.start ].name = private.tasks[ i ][ 4 ][ j ][ 1 ];
      nme = private.start[ #private.start ].name;
      private.start[ #private.start ].prop = ObjGet( nme );
      private.start[ #private.start ].parent = ObjGetRelations( nme ).parent;

    end;
    
  end;
  
  for i = 1, #game.charmonlock.double_hint, 1 do

    for j = 1, 2, 1 do

      table.insert( private.start, {} );
      private.start[ #private.start ].name = game.charmonlock.double_hint[i][j];
      nme = private.start[ #private.start ].name;
      private.start[ #private.start ].prop = ObjGet( nme );
      private.start[ #private.start ].parent = ObjGetRelations( nme ).parent;

    end;
    
  end;

end;
--*********************************************************************************************************************
function public.Load()
  
  if not ng_global.progress[ ng_global.currentprogress ][ "win_charmonlock" ].stage or ng_global.currentprogress == "secret" then
    ng_global.progress[ ng_global.currentprogress ][ "win_charmonlock" ].stage = { 1, 1 };   --[1] - номер стадии, [2] - количество стадий
    ng_global.progress[ ng_global.currentprogress ][ "win_charmonlock" ].complete = 
      { false, false, false, false, false, false, false, false, false, false, false, false };
  end;

  private.complete = ng_global.progress[ ng_global.currentprogress ][ "win_charmonlock" ].complete;

end;
--*********************************************************************************************************************
function public.PreOpen()
  
  cmn.StopAllEnv("assets/levels/common/audio/aud_nearcastle_env");
  cmn.PlayAudio("env", "assets/levels/common/audio/aud_nearcastle_env");

  if ng_global.currentprogress == "std" then
    
    common.SetPreviousRoom( "rm_housegarden" );
    cmn.StartTimePassing("ho_charmonlock");
    
  else 
    
    private.CreateNewObjects();
    
    ng_global.progress[ ng_global.currentprogress ][ "win_charmonlock" ].stage = { 1, 1 };
    ng_global.progress[ ng_global.currentprogress ][ "win_charmonlock" ].complete = 
      { false, false, false, false, false, false, false, false, false, false, false, false };
      
    private.complete = ng_global.progress[ ng_global.currentprogress ][ "win_charmonlock" ].complete;
    
    interface.WidgetSetInput( InterfaceWidget_BtnHint,   true );
    interface.WidgetSetVisible( InterfaceWidget_BtnHint,   true, true );
    
    ObjSet("grm_charmonlock_housegarden",  { event_mdown = "", event_mup = rm_extra.GoToSecretroom, input = 1,
                                             event_menter = function () rm_extra.GateMouseEnter( "charmonlock_housegarden", CURSOR_DOWN, "extra" ); end, 
                                             event_mleave = function () common_impl.GateMouseLeave( "charmonlock_housegarden" ); end });
    
  end;
                                             
   private.DrawScene();                                          
  
  if not cmn.IsEventStart( "win_charmonlock" ) then
    cmn.InitHo( "charmonlock" );
  end;

  local ho_stage = ng_global.progress[ ng_global.currentprogress ][ "win_charmonlock" ].stage;
  
  if not cmn.IsEventDone("win_charmonlock") and ho_stage[1] <= ho_stage[2] then

    local ho_tasks = {};
    interface.TaskPanelTaskAdd( ho_tasks, "eclipse",            1, 0, { { "obj_charmonlock_eclipse" } } );
    interface.TaskPanelTaskAdd( ho_tasks, "letter",             1, 0, { { "obj_charmonlock_letter" } } );
    interface.TaskPanelTaskAdd( ho_tasks, "watch",              1, 0, { { "obj_charmonlock_watch" } } );
    interface.TaskPanelTaskAdd( ho_tasks, "luck",               1, 0, { { "obj_charmonlock_luck" } } );
    interface.TaskPanelTaskAdd( ho_tasks, "music",              1, 0, { { "obj_charmonlock_music" } } );
    interface.TaskPanelTaskAdd( ho_tasks, "aladdin",            1, 0, { { "obj_charmonlock_aladdin" } } );
    interface.TaskPanelTaskAdd( ho_tasks, "magic",              1, 0, { { "obj_charmonlock_magic" } } );
    interface.TaskPanelTaskAdd( ho_tasks, "redhood",            1, 0, { { "obj_charmonlock_redhood" } } );
    interface.TaskPanelTaskAdd( ho_tasks, "hunter",             1, 0, { { "obj_charmonlock_hunter" } } );
    interface.TaskPanelTaskAdd( ho_tasks, "tritonmermaid",      1, 0, { { "obj_charmonlock_tritonmermaid" } } );
    interface.TaskPanelTaskAdd( ho_tasks, "сinderellacarriage", 1, 0, { { "obj_charmonlock_сinderellacarriage" } } );
    interface.TaskPanelTaskAdd( ho_tasks, "clown",              1, 0, { { "obj_charmonlock_clown" } } );

    interface_impl.TaskPanelShow( ho_tasks );

  else
    --public.win();
  end;

end;
--*********************************************************************************************************************
function public.Open()
  
  cmn.PlayAudio("mus", "assets/levels/common/audio/aud_ho_music_theme", 1);
  
  if ng_global.currentprogress == "std" and not ng_global.lvl1_tutor_skip and not(cmn.IsEventDone("tutor_20")) then

    local arrows = { { pos_x = 0, pos_y = 150, ang = 3.14 } };
    cmn.TutorialShow( "tutor_20", 0, 512, 450, arrows );

  end;
  
  local ho_stage = ng_global.progress[ ng_global.currentprogress ][ "win_charmonlock" ].stage;
  if cmn.IsEventDone("win_charmonlock") or ho_stage[1] > ho_stage[2] then
    ObjSet("grm_charmonlock_housegarden",  { input = 0 });
    public.win();
  end;
  
end;
--*********************************************************************************************************************
function public.PreClose()

  interface_impl.TaskPanelHide();
  
  cmn.StopAllMus();
  
  if ng_global.currentprogress == "std" then
  
    common_impl.PlayBackgroundMusic();

    common_impl.StopwatchHoClose();
    common_impl.achiev.stopwatch = {};

    interface.WidgetSetInput( InterfaceWidget_Inventory, true );

    common_impl.PauseAchiev(0);
    
  else 
    
    ObjSet("grm_charmonlock_housegarden",  { input = 0 });
    
    interface.WidgetSetInput( InterfaceWidget_BtnHint,   false );
    interface.WidgetSetVisible( InterfaceWidget_BtnHint,   false, true );
    
  end;
  
  ObjSet("tmr_charmonlock_timer",  { playing = 0 });

end;
--*********************************************************************************************************************
function public.HoComplete()
  
  if ng_global.currentprogress == "std" then

    common_impl.PauseAchiev(1);

    cmn.EndTimePassing("ho_charmonlock");

    if ng_global.progress[ ng_global.currentprogress ][ "win_charmonlock" ].is_hint ~= 1
    and not ng_global.nohintho[ "win_charmonlock" ] then

      ng_global.nohintho[ "win_charmonlock" ] = 1;
      common_impl.CheckAchiev( "ho" );

    end;
    
    if ng_global.playedhops and not ng_global.playedhops[ "win_charmonlock" ] then
      ng_global.playedhops[ "win_charmonlock" ] = 1;
    end;
    
  else 
    
    ObjSet("grm_charmonlock_housegarden",  { input = 0 });
    
  end;
  
end;
--*********************************************************************************************************************
function public.win()
  
  if ng_global.currentprogress == "std" then
    
    cmn.SetEventDone("win_charmonlock");
    cmn.CallEventHandler("win_charmonlock");
    --cmn.GotoRoom("rm_castlehall");
    
    ObjSet("tmr_charmonlock_timer",  { time = 0.3, playing = 1, endtrig = function() cmn.GotoRoom("rm_castlehall"); end });
    
  else 
    
    int_dialog_secretroom.ShowAnim();
    
  end;
  
end;
--*********************************************************************************************************************
function private.DrawScene()
  
  if ng_global.currentprogress == "std" then

    for i = 1, #private.item_names, 1 do
      if private.complete[i] then
        ObjDelete( game.charmonlock.double_hint[ i ][1] );
        ObjDelete( game.charmonlock.double_hint[ i ][2] );
      end;
    end;
    
  else 
    
    for i = 1, #private.item_names, 1 do
      for j = 1, 2, 1 do
        
        ObjMultiSet( { {game.charmonlock.double_hint[ i ][ j ], { alp = 1, input = 1 }},
                       {game.charmonlock.double_hint[ i ][ j ].."_blue", { alp = 0 }} } );
                       
      end;
    end;
    
  end;

end;
--*********************************************************************************************************************
function public.ChooseItem( item_name, ind )

  --DbgTrace( "***ChooseItem" );

  if private.selected_item ~= 0 then

    if private.selected_item ~= ind then

      cmn.PlayAudio("snd", "assets/levels/common/audio/aud_wrong_apply");

      --DbgTrace( "pair don't match" );
      local first_item = public.changed_name[1];

      public.changed_name[1] = "";
      private.selected_item = 0;

      ObjAttach( item_name, "obj_charmonlock_items_fly" );
      ObjMultiSet( { { first_item, { input = false } },
                     { item_name,  { input = false } } } );

      ObjAnimate( first_item.."_blue", "alp", 0, 0, "", { 0, 0, ObjGet( item_name.."_blue" ).alp,    0.3, 0, 0 } );
      ObjAnimate( first_item.."_red",  "alp", 0, 0, "", { 0, 0, 0,    0.3, 0, 1,    0.6, 0, 0 } );

      ObjAnimate( item_name.."_red",   "alp", 0, 0, 
      function ()
        ObjAttach( first_item, "ho_charmonlock" );
        ObjAttach( item_name,  "ho_charmonlock" );
        ObjMultiSet( { { first_item, { input = true } },
                       { item_name,  { input = true } } } );
      end,
      { 0, 0, 0,    0.3, 0, 1,    0.6, 0, 0 } );

    elseif private.selected_item == ind and public.changed_name[1] ~= item_name then

      cmn.PlayAudio( "snd", "assets/levels/common/audio/aud_menter_ho" );

      --DbgTrace( "found right pair" );
      --DbgTrace( "attach "..item_name );

      ObjAttach( item_name, "obj_charmonlock_items_fly" );
      ObjMultiSet( { { public.changed_name[1], { input = false } },
                     { item_name, { input = false } } } );

      private.selected_item = 0;
      public.changed_name[1] = "";
      public.changed_name[2] = "";

      ObjAnimate( item_name.."_blue", "alp", 0, 0, 
      function () 
        private.FounPairAnim( ind );
      end, {
        0,   0, ObjGet( item_name.."_blue" ).alp,
        0.3, 0, 1
      } );

    elseif public.changed_name[1] == item_name then
      
      cmn.PlayAudio( "snd", "assets/levels/common/audio/aud_menter_ho" );

      --DbgTrace( "cancel selection" );
      public.changed_name[1] = "";
      private.selected_item = 0;

      --DbgTrace( "detach "..item_name );
      ObjAttach( item_name, "ho_charmonlock" );
      ObjAnimate( item_name.."_blue", "alp", 0, 0, "", { 0, 0, ObjGet( item_name.."_blue" ).alp,    0.3, 0, 0 } );

    end;

  else 

    cmn.PlayAudio( "snd", "assets/levels/common/audio/aud_menter_ho" );

    --DbgTrace( "first selected" );
    public.changed_name[1] = item_name;
    private.selected_item = ind;

    --DbgTrace( "attach "..item_name );
    ObjAttach( item_name, "obj_charmonlock_items_fly" );
    ObjAnimate( item_name.."_blue", "alp", 0, 0, "", { 0, 0, ObjGet( item_name.."_blue" ).alp,    0.3, 0, 1 } );

  end;

end;
--*********************************************************************************************************************
function private.FounPairAnim( ind )

  --DbgTrace( "***FounPairAnim: ind = "..ind );

  private.selected_item = 0;

  local obj1 = game.charmonlock.double_hint[ ind ][1];
  local obj2 = game.charmonlock.double_hint[ ind ][2];

  local obj1_pos = { ObjGet( obj1 ).pos_x, ObjGet( obj1 ).pos_y };
  local obj2_pos = { ObjGet( obj2 ).pos_x, ObjGet( obj2 ).pos_y };
  local end_pos = common_impl.math.FindSegmentPoint( obj1_pos, obj2_pos, 1 );

  ObjSet( "obj_charmonlock_"..private.item_names[ind], { pos_x = end_pos[1], pos_y = end_pos[2] } );

  ObjAnimate(obj1, "pos_xy", 0, 0, function () private.Fly(ind); end, {
    0,   0, obj1_pos[1], obj1_pos[2],
    0.5, 0, end_pos[1],  end_pos[2]
  } );

  ObjAnimate(obj2, "pos_xy", 0, 0, "", {
    0,   0, obj2_pos[1], obj2_pos[2],
    0.5, 0, end_pos[1],  end_pos[2]
  } );

  ObjAnimate(obj1, "alp", 0, 0, "", { 0, 0, 1,  0.4, 0, 1,    0.6, 0, 0 });
  ObjAnimate(obj2, "alp", 0, 0, "", { 0, 0, 1,  0.4, 0, 1,    0.6, 0, 0 });

end;
--*********************************************************************************************************************
function private.Fly( ind )

  cmn.HoTaskFind( "obj_charmonlock_"..private.item_names[ind], private.item_names[ind], 1 );

  private.complete[ ind ] = true;

end;
--*********************************************************************************************************************
function private.CreateNewObjects()

  local ho_obj = private.start;
  
  for i = 1, #ho_obj, 1 do
    
    if not ObjGet( ho_obj[i].name ) then
      
      local typ = common.GetObjectPrefix( ho_obj[i].name );
      
      if typ == "anm" then
        typ = "anim";
      end;
      
      ObjCreate( ho_obj[i].name, typ );
      ObjSet( ho_obj[i].name, ho_obj[i].prop );
      ObjAttach( ho_obj[i].name, ho_obj[i].parent );

    else 
      
      ObjSet( ho_obj[i].name, ho_obj[i].prop );
      ObjAttach( ho_obj[i].name, ho_obj[i].parent );

    end;

  end;

end;
--*********************************************************************************************************************
