-- name=rm_obb

function public.Init ()
 
  private.DbgTrace("[Init]");

  local screen_size   = cmn.GetScreenSize();
  local field_size    = cmn.GetGameFieldSize();
  local screen_scale  = cmn.GetInterfaceScale();
  local pos_mid       = cmn.CorrectCoords( { pos_x = 512, pos_y = 384 } );
  local scales        = { x = screen_size.w/field_size.w, y = screen_size.h/field_size.h };
  local hub_scales    = { x = ( 1/scales.x ), y = ( 1 / scales.y ) };

  ObjSet( "spr_obb_bg", 
  { 
    pos_x = pos_mid.pos_x,
    pos_y = pos_mid.pos_y, 
    --scale_x = screen_scale, 
    --scale_y = screen_scale, 
    scale_x = (screen_size.w / 1024), 
    scale_y = (screen_size.h / 768),
  } );

  ObjSet( "obj_obb_hub", 
  { 
    inputrect_w = screen_size.w,
    inputrect_h = screen_size.h,
    alp = 0
  } );

  ObjSet( "obj_obb_progress", 
  { 
    pos_x = pos_mid.pos_x,
    pos_y = screen_size.h, 
    scale_x = screen_scale, 
    scale_y = screen_scale,
    --alp = 0   
  } );

  --ObjSet( "grd_obb_progress_bar", 
  --{ 
  --  alp = 0   
  --} );

  ObjSet( "rm_obb",
  {
--    trg_preopen   = private.PreOpen,
--    trg_open      = private.Open,
--    trg_preclose  = private.PreClose,
--    trg_close     = private.Close,

    scale_x       = hub_scales.x, 
    scale_y       = hub_scales.y, 
    inputrect_w   = screen_size.w,
    inputrect_h   = screen_size.h
  } );


  private.need_download_obb = "ready";
  
  private.progress_state = 0;

  private.str_progress = "";

  private.total = 30;
  private.remainingTime = 30;
  private.speed = 10.2;

  private.progress = 0;

end;

function private.DbgTrace ( msg )

  DbgTrace("[menu::obb] "..tostring( msg ) );

end;


--------------------------------------------------------------------
--------------------------------------------------------------------

function public.PreOpen ()

  private.DbgTrace("[PreOpen]");

--  SetSysCursor(true);
--  cmn.SetCursorType( CURSOR_DEFAULT );

  private.need_download_obb = ObbInit();
  private.DbgTrace("[Open] need_download "..tostring(private.need_download_obb));
  if ( private.need_download_obb == "download" ) then

    private.StartAnimShow();

    private.BarUpdate( { total = 0, progress = 0 } );

    private.SubscribeObbDownloadStateChanged();
    private.SubscribeDownloadIsComplete();
    private.SubscribeProgressDownloadObb();

  elseif ( private.need_download_obb == "ready" ) then

    private.show = true;
    private.StartAnimHideByLocalObb();

  elseif ( private.need_download_obb == "error" ) then

    private.StartAnimShow();
    private.DownloadError();

  else

    private.show = true;
    private.StartAnimHide();

  end;


end;

function public.Open ()

  private.DbgTrace("[Open]");

  
end;

function public.PreClose ()

  private.DbgTrace("[PreClose]");

  private.UnsubscribeObbDownloadStateChanged();
  private.UnsubscribeDownloadIsComplete();
  private.UnsubscribeProgressDownloadObb();
  --

end;

function public.Close ()

   private.DbgTrace("[Close]");
--  cmn.SetCursorType( CURSOR_DEFAULT );

end;

--------------------------------------------------------------------
--------------------------------------------------------------------

--подписка на обновление прогресса загрузки
function private.SubscribeProgressDownloadObb ( )

  if not ( private.subscribe_download_obb ) then

    private.subscribe_download_obb = true;

    MsgSubscribe( Event_ObbDownloadProgress, private.UpdateProgress );

  end;

end;

--------------------------------------------------------------------

function private.UnsubscribeProgressDownloadObb ( )

  if ( private.subscribe_download_obb ) then

    MsgUnsubscribe( Event_ObbDownloadProgress, private.UpdateProgress );

  end;

end;

--------------------------------------------------------------------

function private.UpdateProgress ( msg, params )

  private.DbgTrace("[UpdateProgress] ");

  if ( params ) then

    local total = params.total or 0;
    local progress = params.progress or 0;
    local remaining_time = params.remainingTime or 0;
    local download_speed = params.speed or 0;
  
--[[    
    if ( private.str_progress == "..." ) then

      private.str_progress = "";

    else

      private.str_progress = private.str_progress..".";

    end;

    private.str_progress = private.str_progress.." "..progress_files.."/"..total_files.." time: "..remaining_time.." ("..download_speed..")";
    --private.str_progress = "  :  "..progress_files.."/"..total_files.."   time: "..remaining_time.." ms  ("..download_speed.." Kbps)";

]]

    if ( private.progress_state < progress ) then

--      private.progress_state = progress;
--
--      if ( private.str_progress == "..." ) then
--        private.str_progress = "";
--      else
--        private.str_progress = private.str_progress..".";
--      end;

      private.str_progress = "...";
      local str = StringGet( "str_rm_obb_progress_download" )..private.str_progress;

      private.ShowTextDownloadStateChanged( str );

    end;

    private.BarUpdate( { total = total, progress = progress } );

  end;

end; 

function private.ShowTextDownloadStateChanged ( msg )

    local str = msg; --StringGet( "str_rm_obb_progress_download", progress_files, total_files, remaining_time, download_speed );
    --private.str_progress = "  :  "..progress_files.."/"..total_files.."   time: "..remaining_time.." ms  ("..download_speed.." Kbps)";

    ObjSet( "txt_obb_progress_download", { text = str } );

end; 

--------------------------------------------------------------------
--------------------------------------------------------------------

--подписка на завершение загрузки
function private.SubscribeDownloadIsComplete ( )

  if not ( private.subscribe_download_complete ) then

    private.subscribe_download_complete = true;

    MsgSubscribe( Event_ObbDownloadCompleted, private.DownloadIsComplete );

  end;

end;

--------------------------------------------------------------------

function private.UnsubscribeDownloadIsComplete ( )

  if ( private.subscribe_download_complete ) then

    MsgUnsubscribe( Event_ObbDownloadCompleted, private.DownloadIsComplete );

  end;

end;

--------------------------------------------------------------------
function private.DownloadIsComplete ( )

  private.HideProcessingAnim();
  private.UnsubscribeProgressDownloadObb();
  private.UnsubscribeDownloadIsComplete();

end;

--------------------------------------------------------------------
--------------------------------------------------------------------

--подписка на состояние загрузки
function private.SubscribeObbDownloadStateChanged ( )

  if not ( private.subscribe_obb_download_state_changed ) then

    private.subscribe_obb_download_state_changed = true;

    MsgSubscribe( Event_ObbDownloadStateChanged, private.DownloadStateChanged );

  end;

end;

--------------------------------------------------------------------

function private.UnsubscribeObbDownloadStateChanged ( )

  if ( private.subscribe_obb_download_state_changed ) then

    MsgUnsubscribe( Event_ObbDownloadStateChanged, private.DownloadStateChanged );

  end;

end;

--------------------------------------------------------------------
function private.DownloadStateChanged ( msg, params )

  --[[
    stateCode
    state
    table.Push("state", "OBB_STATE_UNKOWN");

    stateMap[1] = "OBB_STATE_IDLE";
    stateMap[2] = "OBB_STATE_FETCHING_URL";
    stateMap[3] = "OBB_STATE_CONNECTING";
    stateMap[4] = "OBB_STATE_DOWNLOADING";
    stateMap[5] = "OBB_STATE_COMPLETED";
    stateMap[6] = "OBB_STATE_PAUSED_NETWORK_UNAVAILABLE";
    stateMap[7] = "OBB_STATE_PAUSED_BY_REQUEST";
    stateMap[8] = "OBB_STATE_PAUSED_WIFI_DISABLED_NEED_CELLULAR_PERMISSION";
    stateMap[9] = "OBB_STATE_PAUSED_NEED_CELLULAR_PERMISSION";
    stateMap[10] = "OBB_STATE_PAUSED_WIFI_DISABLED";
    stateMap[11] = "OBB_STATE_PAUSED_NEED_WIFI";
    stateMap[12] = "OBB_STATE_PAUSED_ROAMING";
    stateMap[13] = "OBB_STATE_PAUSED_NETWORK_SETUP_FAILURE";
    stateMap[14] = "OBB_STATE_PAUSED_SDCARD_UNAVAILABLE";
    
    stateMap[15] = "OBB_STATE_FAILED_UNLICENSED";
    stateMap[16] = "OBB_STATE_FAILED_FETCHING_URL";
    stateMap[17] = "OBB_STATE_FAILED_SDCARD_FULL";
    stateMap[18] = "OBB_STATE_FAILED_CANCELED";
    stateMap[19] = "OBB_STATE_FAILED";
  ]]


  local error = false;
  local state_unkown = false;
  local list_code_error = { 15, 16, 17, 18, 19 };

  if ( params ) and ( params.stateCode ) then

    for i = 1, #list_code_error do

      if ( params.stateCode == list_code_error[i] ) then
        error = true;
        break;
      end;

    end; 

    if ( params.state == "OBB_STATE_UNKOWN" ) then
      state_unkown = true;
    end;

  end;

  private.DbgTrace("[DownloadStateChanged] state= "..tostring( params.state ) );

  if ( IsEditor() or IsCheater() ) then

    if ( state_unkown ) then

      --DbgTrace("[DownloadStateChanged] UNKOWN stateCode= "..tostring( params.stateCode ).." state= "..tostring( params.state ) );

      local id_str = "str_rm_obb_progress_"..tostring( params.state );
      local str_text = "UNKOWN stateCode= "..tostring( params.stateCode ).." "..StringGet( id_str );  

      private.DbgTrace("[DownloadStateChanged] UNKOWN "..tostring( str_text ) );

      local str_text = StringGet( id_str );
      local str_code = params.stateCode;

      private.ShowTextDownloadStateChanged( str_text );

    else

      local id_str = "str_rm_obb_progress_"..tostring( params.state );
      local str_text = StringGet( id_str );
      local str_text = StringGet( id_str );   

      private.DbgTrace("[DownloadStateChanged] STATE text = "..tostring( str_text ) );

      private.ShowTextDownloadStateChanged( str_text );

    end;

  end;


  if ( error ) then

    cmn.WaitingShow();

    ObjSet( "int_control", { pos_z = 1000000 } );
    ObjSet( "hub_int_window", { pos_z = 1000 } );

    local func_common = function ()

      cmn.CallEventHandler( "unsubscribe" );
      int.DialogCommonHide( "information", "downloaderror" );
      --cmn.WaitingHide();
      Quit();

    end;

    local func_accept = function ()

      func_common();

    end;

    local func_reject = function ()

      func_common();

    end;

    local id_str = "str_rm_obb_progress_"..tostring( params.state );
    local str_text = StringGet( id_str );
    local str_code = params.stateCode;

    if ( not ( IsEditor() and IsCheater() )) then
      str_text = "";
      str_code = "";
    end;

    int.DialogCommonShow( "information", "downloaderror", 
    { 
      trg_accept = func_accept, 
      trg_reject = func_reject, 
      state = str_text, 
      state_code = str_code  
    }, nil, nil );
    
  end;

  --private.HideProcessingAnim();
  --private.UnsubscribeProgressDownloadObb();
  --private.UnsubscribeDownloadIsComplete();

end;

--------------------------------------------------------------------
--------------------------------------------------------------------

function private.StartAnimShow (  )

  private.DbgTrace("[StartAnimShow]");
  local name      = "obj_obb_hub";
  local func_anim = function () private.EndAnimShow(); end;
  
  private.show = true;

  ObjAnimate( name, "alp", 0, 0, func_anim, { 0,0,0, 0.6,0,1 } );

  --private.StartTimerProgress();
  private.ShowProcessingAnim();

end;

--------------------------------------------------------------------

function private.EndAnimShow ( )

  private.DbgTrace("[EndAnimShow]");
  local name      = "obj_obb_hub";
  local func_anim = function () private.StartAnimHide(); end;

  ObjSet( name, { input = true } );
  ObjStopAnimate( name, "alp" );

end;

--------------------------------------------------------------------

function private.StartAnimHide ( )

  private.DbgTrace("[StartAnimHide]");
  local func_hide_end = function () private.EndAnimHide(); end;

  local name = "obj_obb_hub";

  if ( private.show ) then

    private.show = false;

    ObjSet( name, { input = false } );
    ObjStopAnimate( name, "alp" );
    ObjAnimate( name, "alp", 0, 0, func_hide_end, { 0,0,1, 0.6,0,0 } );

  else

    func_hide_end();

  end;

end;

function private.StartAnimHideByLocalObb ( )

  private.DbgTrace("[StartAnimHideByLocalObb]");
  local func_hide_end = function () private.EndAnimHide(); end;

  local name = "obj_obb_hub";

  if ( private.show ) then

    private.show = false;

    ObjSet( name, { input = false, visible = false } );
    ObjStopAnimate( name, "alp" );
    ObjAnimate( name, "alp", 0, 0, func_hide_end, { 0,0,0.2, 0.2,0,0 } );

  else

    func_hide_end();

  end;

end;

--------------------------------------------------------------------

function private.EndAnimHide ( )

  private.DbgTrace("[EndAnimHide]");
  local name = "obj_obb_hub";

  ObjStopAnimate( name, "alp" );
  ObjSet( name, { input = false, alp = 0 } );

  private.StopTimerProgress();

  private.DbgTrace("[EndAnimHide] call cmn.GotoRoom( \"rm_intro\" )");
--  room.EventAnimEnd("hide_with_black")
  cmn.GotoRoom( "rm_intro" );

end;

--********************************************************************************************************************
--********************************************************************************************************************

function private.ShowProcessingAnim ()

  -- -- -- 
  local need_start = not ObjGet( "anm_obb_process" ).active;
  local anim_idle = function()
    ObjSet( "anm_obb_process", { 
      animfunc = "loading_edle",
      playing = true
    } );
  end;

  -- -- -- 
  ObjSet( "anm_obb_process", { active = true, visible = true } );

  --ObjAnimate( "grd_obb_progress_bar", "alp", 0, 0, "", { 
  --  0.0, 0, 0.0, 
  --  0.4, 0, 1.0 
  --} );

  if need_start then 

    local txt_progress = "txt_obb_progress_download";
    AnimPlay( "anm_obb_process", "loading_start", anim_idle );
    ObjAnimate( txt_progress, "alp", 0, 0, "", { 
      0.0, 0, 0.0, 
      0.4, 0, 1.0 
    } );

  else 

    anim_idle();

  end;

end;

--********************************************************************************************************************

function private.HideProcessingAnim ()

  local need_hide = ObjGet( "anm_obb_process" ).active;
  local on_end = function()

    ObjSet( "anm_obb_process", { active = false, visible = false } );
    private.StartAnimHide();

  end;
  -- -- -- 
  if need_hide then 

    local txt_progress = "txt_obb_progress_download";
    AnimPlay( "anm_obb_process", "loading_end", on_end );
    ObjAnimate( txt_progress, "alp", 0, 0, "", { 
      0.0, 0, 1.0, 
      0.4, 0, 0.0 
    } );
    ObjAnimate( "obj_obb_progress_bar", "alp", 0, 0, "", { 
      0.0, 0, 1.0, 
      0.4, 0, 0.0 
    } );
  else 

    on_end();

  end;  

end;

--------------------------------------------------------------------

function private.BarUpdate ( stats )

  private.bar_time = 0.1;

  local percent = 0;
  local bar = {};

  -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- 
  if ( stats.progress or stats.total ) then

    if not ( stats.total < stats.progress ) then

      --private.stats.min   = 0;
      --private.stats.progress       = stats.progress      or private.stats.progress;
      --private.stats.total   = stats.total  or private.stats.total;

      --private.stats.exp_min = 0;

      local current = stats.progress; --private.stats.progress - private.stats.min;
      local need    = stats.total; --private.stats.total - private.stats.min;

      ObjSet( "txt_obb_progress_bar", { param0 = current, param1 = need } );

      --percent = ( private.stats.progress - private.stats.min ) / ( private.stats.total - private.stats.min );

      if ( not ( stats.progress == 0 )) and ( not ( stats.total == 0 ) ) then
        percent = ( stats.progress ) / ( stats.total );
      end;

      local full_bar  = -130;  -- private.full_bar;
      local empty_bar  = -393;
      local diff_bar  = math.abs( math.abs( full_bar ) - math.abs( empty_bar ) );
      local value_pos = empty_bar + ( diff_bar * percent );

      bar = 
      {
        name              = "grd_obb_progress_bar_fill",
        croprect_x_start  = 0,
        croprect_x_end    = 0,
        pos_x_start       = empty_bar,
        pos_x_end         = empty_bar,
      };

      if ( value_pos > full_bar ) then

        value_pos = full_bar;

      end;

      if ( value_pos < empty_bar ) then

        value_pos = empty_bar;

      end;


      local bar_time  = private.bar_time;
      local params    = ObjGet( bar.name );

      --bar.croprect_x_start = params.croprect_x;
      --bar.croprect_x_end   = full_bar - ( full_bar * percent );

      bar.pos_x_start =   params.pos_x;
      --bar.pos_x_end   = - bar.croprect_x_end - 128;
      bar.pos_x_end   = value_pos;

--      if ( bar.croprect_x_end > full_bar ) then
--        bar.croprect_x_end = full_bar;
--      end;

      private.BarExpAnim( bar );

    end;

  end;

end;

-- анимация заполнения "бар"-панели
function private.BarExpAnim ( bar_data, event_end )

  local bar_time = private.bar_time;

  local func = function ()

    if ( event_end ) then

      event_end();

    end;

  end;

  --ObjAnimate( bar_data.name, "croprect_x", 0, 0, func, 
  --{ 
  --  0,        2, bar_data.croprect_x_start,
  --  bar_time, 2, bar_data.croprect_x_end
  --} );

  ObjAnimate( bar_data.name, "pos_x", 0, 0, "", 
  { 
    0,        2, bar_data.pos_x_start,
    bar_time, 2, bar_data.pos_x_end
  } );

  local name_over = bar_data.name.."_over";
  local alp_over  = ObjGet( name_over ).alp;

  --ObjAnimate( name_over, "alp", 0, 0, "", 
  --{ 
  --  0,              0, alp_over,
  --  bar_time / 2,   0, 1,
  --  bar_time,       0, 0
  --} );

end;

--********************************************************************************************************************
--********************************************************************************************************************
-- function ****** Тестовый функционал ****** () end
--********************************************************************************************************************
--********************************************************************************************************************

--------------------------------------------------------------------
function private.StartTimerProgress ( )

  private.DbgTrace("[StartTimerProgress]");
  local tmr_progress = "tmr_obb_progress";
  ObjSet( tmr_progress, { time = 1,  endtrig = private.UpdateTimerProgress, playing = true } );

  --private.ShowProcessingAnim();

end; 

--------------------------------------------------------------------
function private.StopTimerProgress ( )

  private.DbgTrace("[StopTimerProgress]");
  local obj_progress = "txt_obb_progress";
  local tmr_progress = "tmr_obb_progress";

  ObjSet( obj_progress, { text = "" } );
  ObjSet( tmr_progress, { time = 0,  endtrig = "", playing = false } );

end; 

--------------------------------------------------------------------
function private.UpdateTimerProgress ( )

  private.DbgTrace("[UpdateTimerProgress] "..private.progress );

  if ( private.total > private.progress ) then
  
    private.progress = private.progress + 1;

    local txt_progress = "txt_obb_progress";
    local tmr_progress = "tmr_obb_progress";

    ObjSet( txt_progress, { text = private.progress } );

    ObjSet( tmr_progress, { time = 0.5, endtrig = private.UpdateTimerProgress, playing = true } );

    --if ( private.str_progress == "..." ) then
    --  private.str_progress = "";
    --else
    --  private.str_progress = private.str_progress..".";
    --end;
    --ObjSet( "txt_obb_progress_download", { text = "Downloading files "..private.str_progress } );



    --private.UpdateProgress( nil, { total = private.total, progress = private.progress, remainingTime = math.random( 1, 9 ), speed = (( math.random( 10, 90 ) )/ 3) } );

    --if ( private.total == private.progress ) then
    --  private.UpdateProgress( nil, { total = 0, progress = 0, remainingTime = math.random( 1, 9 ), speed = (( math.random( 10, 90 ) )/ 3) } );
    --end;
    --private.UpdateProgress( nil, { total = 0, progress = 10} );


  else

    private.StopTimerProgress();
--    private.StartAnimHide();

    private.HideProcessingAnim();

  end;

end; 
--------------------------------------------------------------------


--------------------------------------------------------------------
--------------------------------------------------------------------
